//=======================================================================
//
// TealPrint Launch Codes
//
// Call TealPrint (appl,TlPr) as an external
// application passing parameters in the
// appropriate "params" structure
//
//=======================================================================

	enum {
		PRINT_CMD_SystemInfo = 35557,
		PRINT_CMD_GetSettings,
		PRINT_CMD_SetSettings,

		PRINT_CMD_PrintObject,
		PRINT_CMD_PrintClipboard,

		PRINT_CMD_ChooseSettings,
		PRINT_CMD_ChooseOptions,
		PRINT_CMD_ChooseFont,
		PRINT_CMD_ChooseDriver,
		PRINT_CMD_ChoosePaper,
		PRINT_CMD_ChooseGfxMode,
		PRINT_CMD_ChooseTxtMode,

		PRINT_CMD_CallDriver,

		PRINT_CMD_QuickPrintText,		// obsolete... instant is better
		PRINT_CMD_QuickPrintWindow,

		PRINT_CMD_InstantPrintText,
		PRINT_CMD_InstantPrintWindow
	};

	//================================================================
	//
	// Structures and defines
	//
	//================================================================

	//------------------------------------------
	//
	// Printing flag bits
	//
	// Special note:  When landscape printing
	// is used, object positioning and sizing
	// are sideways too.
	//
	//------------------------------------------

	// General options

	#define PRINT_OUTFLAG_center_x		0x0001	// center horizonally
	#define PRINT_OUTFLAG_center_y		0x0002	// center vertically
	#define PRINT_OUTFLAG_flip_x			0x0004	// mirror horizontally
	#define PRINT_OUTFLAG_flip_y			0x0008	// mirror vertically
	#define PRINT_OUTFLAG_landscape		0x0010	// print in landscape mode
	#define PRINT_OUTFLAG_smoothing		0x0020	// print with corner-smoothing of jaggies
	#define PRINT_OUTFLAG_noeject			0x0040	// eject paper after printing (form feed)
	#define PRINT_OUTFLAG_invert_colors	0x0080	// print inverted images

	// Special options

	#define PRINT_OUTFLAG_integer_scale	0x0100	// make sure image is integer scaled for even lines
	#define PRINT_OUTFLAG_ignore_aspect	0x0200	// allow stretching of image out of aspect
	#define PRINT_OUTFLAG_no_wordbreak	0x0400	// allow line breaks anywhere
	#define PRINT_OUTFLAG_no_wordwrap	0x0800	// truncate long lines; don't wrap

	#define PRINT_OUTFLAG_ignore_h_margins 0x1000	// ignore margins when printing
	#define PRINT_OUTFLAG_ignore_v_margins 0x2000	// ignore margins when printing

	// Choose of of these for how to scale/place printable area

	#define PRINT_OUTFLAG_size_percent	   0x0000	// final size based on 75dpi palm screen res
	#define PRINT_OUTFLAG_size_pixels	   0x4000	// actual size pixels
	#define PRINT_OUTFLAG_size_mils		   0x8000	// actual size in thousandsths of an inch
	#define PRINT_OUTFLAG_size_multiple	   0x10000L	// multiple of printer resolution in percent
	#define PRINT_OUTFLAG_size_fraction    0x20000L	// fraction of printable paper area in percent
	#define PRINT_OUTFLAG_size_chars		   0x40000L	// actual size characters

	#define PRINT_OUTMASK_size				   0x7c000L
	#define PRINT_OUTSHFT_size					14

	// Text alignment flags

	#define PRINT_OUTFLAG_justify_left		0x000000L
	#define PRINT_OUTFLAG_justify_center	0x100000L
	#define PRINT_OUTFLAG_justify_right		0x200000L

	#define PRINT_OUTMASK_justify				0x300000L
	#define PRINT_OUTSHFT_justify				20

	#define PRINT_OUTFLAG_relative_left		0x0400000L
	#define PRINT_OUTFLAG_relative_right	0x0800000L
	#define PRINT_OUTFLAG_relative_top		0x1000000L
	#define PRINT_OUTFLAG_relative_bottom	0x2000000L

	#define PRINT_OUTMASK_relative			0x3c00000L
	#define PRINT_OUTSHFT_relative			22

	#define PRINT_OUTFLAG_all_or_nothing	0x4000000L	// if set, this object will not be draw if it extends outside its parent object
																		// typically due to use of object_relative flags

	#define PRINT_OUTFLAG_noflush				0x8000000L	// flag to spool driver that we're a continuance

	#define PRINT_OUTFLAG_relative_width  0x10000000L	// width relative (percentage, pixel, or character) from relative_bound
	#define PRINT_OUTFLAG_relative_height 0x20000000L	// height relative to relative_bound

	#define PRINT_OUTFLAG_offset_right	  0x40000000L	// align right edge by subtracting final width from x bound
	#define PRINT_OUTFLAG_offset_bottom	  0x80000000L	// align bottom edge by subracting final height from y bound

	//------------------------------------------
	//
	// Font-Handling defines
	//
	//------------------------------------------

	// Style bits

	#define FONT_FLAG_italics		1
	#define FONT_FLAG_underline	2
	#define FONT_FLAG_bold			4
	#define FONT_FLAG_serif			8
	#define FONT_FLAG_script		16
	#define FONT_FLAG_symbol		32
	#define FONT_FLAG_fixedspace	64

	// Names of built-in fonts

	#define FONT_FACE_default	"Default"
	#define FONT_FACE_palm0		"Palm 0"
	#define FONT_FACE_palm1		"Palm 1"
	#define FONT_FACE_palm2		"Palm 2"
	#define FONT_FACE_palm7		"Palm 7"

	// Font request structure

	typedef struct {
		char face[32];				// Desired face name or ""
		long flags;
		int size;					// Desired pt size
		int min_size;				// Min size or 0
		int max_size;				// Max size or 0
		int pad;
	} FontSpec;

	//------------------------------------------
	//
	// Print State Structure
	// returned with PRINT_CMD_GetState call
	//
	//------------------------------------------
	
	typedef struct {

		int	version;				// set to zero
	
		int	print_mode;			// PRINT_MODE_text or PRINT_MODE_graphic

		long	graphic_flags;
		int	graphic_x;
		int	graphic_y;
		int	graphic_w;
		int	graphic_h;
		FontSpec graphic_font;

		long	text_flags;
		int	text_x;
		int	text_y;
		int	text_w;
		int	text_h;
	
	} PrintSettings;

	enum {
		PRINT_MODE_text,
		PRINT_MODE_graphic
	};

	//------------------------------------------
	//
	// Data color modes tealprint
	//
	//------------------------------------------

	#define PRINT_GFXMODE_1bit  			0x0001	// data fmt 1 bit-per-pixel msb to lsb
	#define PRINT_GFXMODE_2bitpalette	0x8000	// data fmt 2-bit palette
	#define PRINT_GFXMODE_8bitpalette	0x8001	// data fmt 8-bit palette

	//================================================================
	//
	// Parameters for calls to print system
	// 
	//================================================================

	typedef struct {				// Simple parameters
		long	retval;
	} SimpleParams;

	typedef struct {				// Get system information
		long	retval;
		int	version;
		int 	revision;
		char	name[32];
		long  flags;
	} SystemInfoParams;

	typedef struct {
		long	retval;
		PrintSettings *settings;
	} SetSettingsParams;

	typedef struct {
		long	retval;
		PrintSettings *settings;
	} GetSettingsParams;

	//------------------------------------------
	//
	// Print Interface Routines
	//
	//------------------------------------------

	// OBJECT - common parameters at the top of each function

	typedef struct PrintObject {
			  
		int	type;
		int   dx,dy,dw,dh;
		long  print_flags;
		struct PrintObject *ref_obj;	// for relative positioning
		void	*system_data;				// reserved for system use

	} PrintObject;

	// BITMAP - easily print a subrectangle from a window

	typedef struct {

		int	type;					// set to PRINT_OBJECT_bitmap
		int	dx,dy,dw,dh;
		long  print_flags;
		PrintObject *ref_obj;	// for relative positioning
		void	*system_data;		// reserved for system use

		int	sx,sy,sw,sh;
		WindowType *easy_window;

		void  (*callback_drawfunc)( WindowType *window, int x,int y,int w,int h, void *app_data );
		void   *callback_data;

	} PrintBitmapObject;

	// TEXT - print a block of text

	typedef struct PrintTextObject {

		int	type;					// set to PRINT_OBJECT_string
		int	dx,dy,dw,dh;
		long  print_flags;
		PrintObject *ref_obj;	// for relative positioning
		void	*system_data;		// reserved for system use

		FontSpec *text_font;		// font data to override inherited font or 0

		long  text_length;

		char *easy_text;
		void  (*callback_copyfunc)( char *buffer, long offset, long count, void *app_data );
		void   *callback_data;

		long	 chars_used;							// returned count of characters actually used
		struct PrintTextObject *preflow_ptxt;	// pointer to previous ptxt object we're overflowing from or 0

	} PrintTextObject;

	// RECTANGLE - print a rectangle

	typedef struct {

		int	type;					// set to PRINT_OBJECT_rectangle
		int	dx,dy,dw,dh;
		long  print_flags;
		PrintObject *ref_obj;	// for relative positioning
		void	*system_data;		// reserved for system use

		long  mode_flags;
		long  line_color;			// RGBA color
		long	line_pattern;
		int	line_weight;		// weight in points, 0 for finest
		int	corner_radius;		// corner radius in points
		long	fill_pattern[2];

	} PrintRectangleObject;

	// VLINE

	typedef struct {

		int	type;					// set to PRINT_OBJECT_vline
		int	dx,dy,dw,dh;
		long  print_flags;
		PrintObject *ref_obj;	// for relative positioning
		void	*system_data;		// reserved for system use

		long  mode_flags;
		long  draw_color;			// RGBA color
		long	line_pattern;
		int	line_weight;		// weight in points, 0 for finest

	} PrintVLineObject;

	// HLINE

	typedef struct {

		int	type;					// set to PRINT_OBJECT_hline
		int	dx,dy,dw,dh;
		long  print_flags;
		PrintObject *ref_obj;	// for relative positioning
		void	*system_data;		// reserved for system use

		long  mode_flags;
		long  draw_color;			// RGBA color
		long	line_pattern;
		int	line_weight;		// weight in points, 0 for finest

	} PrintHLineObject;

	// PAGE - a laid out page containing other objects

	typedef struct {

		int	type;
		int   dx,dy,dw,dh;
		long  print_flags;
		PrintObject *ref_obj;	// for relative positioning
		void	*system_data;		// reserved for system use

		int	num_objects;
		PrintObject **objects;	// array of pointers print objects

	} PrintPageObject;

	// Print Params

	typedef struct {

		long	retval;
		PrintObject	  *po;		// object to print			  
		PrintSettings *ps;		// or null to use defaults

	} PrintObjectParams;

	typedef struct {

		long	retval;
		PrintSettings *ps;		// or null to use defaults

	} PrintClipboardParams;

	// Object enum

	enum {
		PRINT_OBJECT_page,
		PRINT_OBJECT_simple_string,
		PRINT_OBJECT_virtual_string,
		PRINT_OBJECT_simple_bitmap,
		PRINT_OBJECT_virtual_bitmap,
		PRINT_OBJECT_rectangle,
		PRINT_OBJECT_vline,
		PRINT_OBJECT_hline
	};

	//------------------------------------------
	//
	// Settings Routine
	//
	// this is often used as a configuration
	// screen just prior to printing, as 
	// all global options can be set from here.
	//
	//------------------------------------------

	typedef struct {
		long	retval;
		long  flags;
		PrintSettings *ps;		// pointer to settings to modify or 0 to modify internal defaults
	} ChooseSettingsParams;

	#define PRINT_CHOOSE_SET_nodriver  	0x0002	// don't allow setting of driver
	#define PRINT_CHOOSE_SET_nopaper		0x0004	// don't allow setting of paper sizes
	#define PRINT_CHOOSE_SET_nogfxmode 	0x0008	// don't allow setting of gfx mode
	#define PRINT_CHOOSE_SET_notxtmode 	0x0010	// don't allow setting of text mode
	#define PRINT_CHOOSE_SET_noptions	0x0040	// don't link to options flags
	#define PRINT_CHOOSE_SET_nobounds  	0x0080	// don't allow setting of bounds rectangles
	#define PRINT_CHOOSE_SET_nogfxtxt 	0x0100	// don't allow choosing text vs graphics mode

	#define PRINT_CHOOSE_SET_print 		0x0100	// show 'print' button instead of 'ok'

	//------------------------------------------
	//
	// Options Routines
	//
	//------------------------------------------

	typedef struct {
		long	retval;
		long  flags;
		PrintSettings *ps;		// pointer to settings to modify or 0 to modify internal defaults
	} ChooseOptionsParams;

	#define PRINT_CHOOSE_OPTION_print 	0x0100	// show 'print' button instead of 'ok'

	typedef struct {
		long	retval;
		long  flags;
		FontSpec *font;				// pointer to font spec to modify or 0 to modify internal defaults
		int   x_dpi, y_dpi;			// printer dpi or 0 to fetch current
	} ChooseFontParams;

	typedef struct {
		long	retval;
		long	command;
		void	*data;
	} CallDriverParams;

	//================================================================
	//
	// Union for parameters to pass to TealPrint with corresponding
	// launch code.
	//
	//================================================================

	typedef union {
		struct generic_params {
			long retval;
			void *p1,*p2,*p3,*p4,*p5,*p6,*p7,*p8,*p9,*p10,*p11,*p12;
		};

		SystemInfoParams		system_info;
		GetSettingsParams		get_settings;
		SetSettingsParams		set_settings;
		PrintClipboardParams print_clipboard;
		PrintObjectParams   	print_object;
		ChooseSettingsParams choose_settings;
		ChooseOptionsParams  choose_options;
		ChooseFontParams		choose_font;
		SimpleParams			choose_driver;
		SimpleParams			choose_paper;
		SimpleParams			choose_txtmode;
		SimpleParams			choose_gfxmode;
		CallDriverParams  	call_driver;

		char *quick_text;
		WinPtr quick_window;

		char instant_text[1];
		WindowType instant_window;
	} PrintParamsType;

	
	
//=======================================================================
//
// Driver Launch Codes
//
//=======================================================================

	enum {
		DRIVER_CMD_ShowInfo			= 34560,
		DRIVER_CMD_GetDriverInfo,
		DRIVER_CMD_GetGModeInfo,		
		DRIVER_CMD_GetTModeInfo,		
		DRIVER_CMD_GetPaperInfo,		
		DRIVER_CMD_SetPaperInfo,		
		DRIVER_CMD_SetSettings,		
		DRIVER_CMD_EjectPaper,		
		DRIVER_CMD_PrintBitmap,		
		DRIVER_CMD_PrintText,		
		DRIVER_CMD_HardReset,		// slow. can be fairly involved
		DRIVER_CMD_SoftReset			// quick. safe to do before printing	
	};

	#define MAX_PIXEL_BUFFER	1024
	#define MAX_TEXT_BUFFER		1024
	#define MAX_PAPER_SIZES		  20
	#define MAX_GFX_MODES 		  20
	#define MAX_TXT_MODES 		  20
	
	//------------------------------------------
	//
	// Structure for DRIVER_CMD_GetDriverInfo
	//
	//------------------------------------------
	
	typedef struct {
	
		long	retval;
		char  version;
		char  revision;
	
		char *name;	 		// null-terminated driver name
	
		unsigned long flags;		// set unused to 0
	
		int	num_gfxmodes;		// number of supported graphics modes or 0
		int	num_txtmodes;		// number of supported text modes or 0
		int	num_paper_sizes;	// number of standard paper sizes
	
	
	} DriverInfoType;
	
	#define DRIVER_FLAG_has_settings		  0x0001	// set if driver can respond to SetSettings call
	#define DRIVER_FLAG_has_info			  0x0002	// set if driver can respond to ShowInfo call
	#define DRIVER_FLAG_supports_graphics 0x0004
	#define DRIVER_FLAG_supports_text	  0x0008
	#define DRIVER_FLAG_landscape_text	  0x0010
	#define DRIVER_FLAG_supports_color_graphics 0x0020
	
	//------------------------------------------
	//
	// Structure for DRIVER_CMD_GetModeInfo
	//
	//------------------------------------------
	
	typedef struct {
	
		long	retval;
		int 	mode_number;				// passed in by caller (yuck!)
	
		char *name;							// null-terminated mode name
	
		unsigned long flags;				// set unused to 0
		unsigned long supported_modes;
	
		int	x_pixels;
		int	y_pixels;
		int	x_dpi;
		int	y_dpi;
	
	} DriverGModeInfoType;
	
	// Data color modes driver may be called with

	#define DRIVER_GFXMODE_1bit  		0x0001	// data fmt 1 bit-per-pixel msb to lsb
	#define DRIVER_GFXMODE_8bitweb	0x0002	// data fmt 8-bit web palette
	#define DRIVER_GFXMODE_8bitgrey	0x0004	// data fmt 8-bit greyscale value
	#define DRIVER_GFXMODE_24bitrgb	0x0008	// data fmt 24-bit rgb values
	
	//------------------------------------------
	//
	// Structure for DRIVER_CMD_GetModeInfo
	//
	//------------------------------------------
	
	typedef struct {
	
		long	retval;
		int mode_number;					// passed in by caller (yuck!)
	
		char *name;							// null-terminated mode name
	
		unsigned long flags;				// set unused to 0
		unsigned long supported_modes;
	
		int	x_cpi;
		int	y_cpi;
	
	
	} DriverTModeInfoType;

	#define DRIVER_TXTMODE_ascii		  		0x0001	// basic ascii text

	#define DRIVER_TXTFLAG_landscape  		0x0002	// supports landscape text

	#define DRIVER_GFXMODE_normal				0			// 0  = white
	#define DRIVER_GFXMODE_inverted			0xff		// ff = white
	#define DRIVER_GFXMODE_palmcolor			1			// 8-bit std palm palette

	//------------------------------------------
	//
	// Structure for DRIVER_CMD_GetPaperInfo
	//
	//------------------------------------------
	
	typedef struct {
	
		long	retval;
		int 	paper_number;		// passed in by caller (yuck!)
	
		long	flags;
		char *name;				// null-terminated paper size name
	
		int	w_mils_tot; 	// paper size
		int	h_mils_tot;
	
		int	t_mils_cur;		// current margin
		int	b_mils_cur;
		int	l_mils_cur;
		int	r_mils_cur;

		int	t_mils_min; 	// unprintable margin
		int	b_mils_min; 
		int	l_mils_min;	
		int	r_mils_min;	

		int	t_mils_def; 	// default margin
		int	b_mils_def;
		int	l_mils_def;
		int	r_mils_def;
	
	} DriverPaperInfoType;

	//------------------------------------------
	//
	// Structure for DRIVER_CMD_PrintBitmap
	//
	//------------------------------------------
	
	typedef struct {
		long	retval;
		int width;
		int height;
		int blank_lines;
		int blank_cols;
		void (*getraster_func)( unsigned char **targ, int line, int offset, int count, void *data, unsigned char bgcolor );
		void *app_data;
		long print_flags;
		int  mode_number;
		int  paper_number;
	} DriverPrintBitmapType;
	
	#define DRIVER_OUTFLAG_noeject	0x0001
	#define DRIVER_OUTFLAG_noflush	0x0002

	//------------------------------------------
	//
	// Structure for DRIVER_CMD_PrintText
	//
	//------------------------------------------
	
	typedef struct {
		long	retval;
		int cols;
		int lines;
		int blank_lines;
		int blank_cols;
		void (*getline_func)( char **text, int line, int *offset, int *count, void *data );
		void *app_data;
		long print_flags;
		int  mode_number;
		int  paper_number;
	} DriverPrintTextType;
	
	
	
	
	
	
	
	
	
	
